package com.ruoyi.web.controller.app_course;


import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.RestResponse;
import com.ruoyi.course.domain.Member;
import com.ruoyi.course.service.ConsumerCourseServiceImpl;
import com.ruoyi.course.service.MemberServiceImpl;
import com.ruoyi.search.domain.MemberUser;
import com.ruoyi.search.service.MemberUserServicelmpl;
import com.ruoyi.shop.domain.ShopOrder;
import com.ruoyi.shop.domain.ShopOrderItem;
import com.ruoyi.shop.service.impl.ShopOrderItemServiceImpl;
import com.ruoyi.shop.service.impl.ShopOrderServiceImpl;
import com.ruoyi.web.util.PayUtil;
import com.ruoyi.web.util.SnowFlake;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author xhj
 * @since 2022-09-23
 */
@Api(tags = "会籍", description = "提供会籍相关的API")
@RestController
@RequestMapping("/api/member")
public class
MemberAppController {

    @Autowired
    private MemberServiceImpl memberService;
    @Autowired
    private MemberUserServicelmpl memberUserServicel;
    @Resource
    private ShopOrderServiceImpl orderService;
    @Resource
    private ShopOrderItemServiceImpl itemService;
    @Resource
    private ConsumerCourseServiceImpl consumerCourseService;

    /**
     * app服务-本店会籍类型列表
     * @param member
     * @return
     * @author xn
     */
//    @ApiOperation("获取本店会籍列表")
    @PostMapping("/getMembers")
    public RestResponse getMembers(@RequestBody Member member) {
        List<Member> list = memberService.list(new QueryWrapper<Member>()
                .eq("strore_id",member.getStroreId())
                .eq("releases",1)
        );
        return  new RestResponse().setData(list);
    }

    /**
     * app服务-会籍详情
     * @param member
     * @return
     * @author xn
     */
//    @ApiOperation("会籍详情")
    @PostMapping("/getOne")
    public RestResponse getOne(@RequestBody Member member) {
        Member m = memberService.getById(member.getMemberId());
        return  new RestResponse().setData(m);
    }

    /**
     * 数据字典
     * @param member
     * @return
     */
//    @ApiOperation("会籍数据字典")
    @PostMapping(value = "/getList")
    public RestResponse findList1(@RequestBody Member member) {
        List<Member> list =memberService.findList(member);
        return new RestResponse().setSuccess(true).setMessage("成功").setData(list);
    }

    /**
     * 购买会籍下订单
     * @param memberUser
     * @param request
     * @return
     */
//    @ApiOperation("购买会籍")
    @PostMapping("/createOrderForMember")
    @Transactional(rollbackFor = RuntimeException.class)
    public RestResponse createOrderForMember(@RequestBody MemberUser memberUser, HttpServletRequest request) throws Exception{
        Long userId= Long.parseLong(request.getAttribute("userId").toString());
        //判断是否已经在籍
        LocalDateTime dateTime= DateUtil.parseLocalDateTime(DateUtil.format(DateUtil.tomorrow(),"yyyy-MM-dd 00:00:00"));
        List<MemberUser> list=memberUserServicel.list(new QueryWrapper<MemberUser>()//DateUtil.now()
                .eq("member_id",memberUser.getMemberId())
                .eq("consumer_id",userId)
                .last(" and (validity_time > now() or come_time ='"+dateTime+"')"));//截止日期为今天之后（未到期） 或 开始日期在明天（今天办的）
        if (list.size()>0){
            if (list.get(0).getComeTime().equals(dateTime)){
                return new RestResponse().setSuccess(false).setMessage("您已成功办理会籍，生效时间为"+DateUtil.tomorrow().toString("yyyy-MM-dd 00:00:00")+"!请勿重复办理！");
            }else {
                return new RestResponse().setSuccess(false).setMessage("会籍存续中，请勿重复办理！");
            }
        }
        //价格/月 * 有效期(月)
        BigDecimal price= memberUser.getMemberPrice().multiply(new BigDecimal(memberUser.getValidity()));
        //生成订单（支付中）
        ShopOrder order=new ShopOrder();
        order.setOrderSn(SnowFlake.getOrder()+"");//订单号
        order.setBusinessId(memberUser.getStroreId());//商户
        order.setIdUser(userId);//客户id
        order.setMobile(request.getAttribute("phone").toString());//收件人电话（客户电话）
        order.setCreateTime(DateUtil.parseLocalDateTime(DateUtil.now(),"yyyy-MM-dd HH:mm:ss"));
        order.setOrderType(2);//订单类型：1.商品 2.会籍 3.课程
        order.setPayType("微信支付");
        order.setTradeAmount(price);//TradeAmount交易金额

        orderService.save(order);

        //生成订单详情
        ShopOrderItem item=new ShopOrderItem();
        item.setIdOrder(order.getId());//订单id
        item.setIdMember(memberUser.getMemberId());//会员id
        item.setTotalPrice(price);//总价
        item.setThumbnail(memberUser.getMemberPicture());//缩略图
        item.setType("2");//1 商品 2 会籍 3  课程
        item.setMemberType(memberUser.getMemberName());//会籍类型=会籍名称
        item.setMemberValidity(memberUser.getValidity());//会籍有效期（月）
        item.setBrief(memberUser.getMemberContext());//简介=会籍内容
        item.setMemberRecommend(memberUser.getRecommend());//推荐人
        itemService.save(item);
        //单位： 元 转换成 分
        BigDecimal total=price.multiply(new BigDecimal(100));
        Map map=new HashMap();

        //0元
        if (total.equals(BigDecimal.ZERO)){
            //支付成功 生成会籍

            map.put("timeStamp","1");
            return new   RestResponse().setSuccess(true).setMessage("成功").setData(map);
        }
        map= PayUtil.creatOrder(total.setScale(1, RoundingMode.HALF_UP).intValue(),order.getOrderSn());
        return new   RestResponse().setSuccess(true).setMessage("成功").setData(map);
    }

    /**
     * 我的会籍
     * 存续的
     * @return
     */
//    @ApiOperation("我的会籍")
    @PostMapping("/myMembers")
    public RestResponse myMembers(HttpServletRequest request){
        Long userId= Long.parseLong(request.getAttribute("userId").toString());
        List<MemberUser> list=memberUserServicel.list(new QueryWrapper<MemberUser>()
                .eq("consumer_id",userId)
                .last("and validity_time > now()"));
        return new RestResponse().setSuccess(true).setData(list);
    }

   //课程表与会籍相关联


}

