package com.ruoyi.web.controller.app_shop;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.RestResponse;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginBody;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.exception.user.CaptchaException;
import com.ruoyi.common.exception.user.CaptchaExpireException;
import com.ruoyi.common.exception.user.UserPasswordNotMatchException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.ip.IpUtils;
import com.ruoyi.framework.config.JwtUtil1;
import com.ruoyi.framework.manager.AsyncManager;
import com.ruoyi.framework.manager.factory.AsyncFactory;
import com.ruoyi.framework.security.context.AuthenticationContextHolder;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.shop.domain.Consumer;
import com.ruoyi.shop.domain.YjMerchant;
import com.ruoyi.shop.service.ConsumerService;
import com.ruoyi.shop.service.impl.YjMerchantServiceImpl;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.system.service.ISysUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.TimeUnit;
@Api(tags = "登录与验证")
@RestController
@RequestMapping("/api")
public class LoginController {


    @Resource
    private ConsumerService service;
    @Autowired
    private ISysUserService userService;
    @Resource
    JwtUtil1 jwtUtil1;
    @Autowired
    private TokenService tokenService;
    @Resource
    RedisTemplate<String,Object> redisTemplate;

    @Autowired
    private RedisCache redisCache;
    @Autowired
    private ISysConfigService configService;
    @Resource
    private AuthenticationManager authenticationManager;


    @ApiOperation("登录")
    @PostMapping("/login")
    public AjaxResult login(@RequestBody LoginBody loginBody){

        AjaxResult ajax = AjaxResult.success();
        if (StrUtil.isEmpty(loginBody.getRegisterId())){
            ajax.put(Constants.FAIL, "设备id异常");
            return ajax;
        }
        boolean captchaEnabled = configService.selectCaptchaEnabled();
        // 验证码开关
        if (captchaEnabled)
        {
            validateCaptcha(loginBody.getAccount(),loginBody.getCode(),loginBody.getUuid());
        }

        //用户名是否存在
        SysUser user = userService.selectUserByUserName(loginBody.getAccount());

        //未注册
        if(ObjectUtil.isEmpty(user) ){
            //注册并登录
            //用户账号 昵称 门店 密码 头像
            // registerId 地址
            // 必填 user_name nick_name merchant_id
            user=new SysUser();
            user.setUserName(loginBody.getAccount());
            user.setNickName("用户"+loginBody.getAccount());
            user.setPassword(SecurityUtils.encryptPassword(loginBody.getPassword()));
            user.setRegisterId(loginBody.getRegisterId());
            user.setVisitStore(getIndex(user).get("id"));
            Long[] ros={107l};//用户角色
            user.setRoleIds(ros);
            userService.insertUser(user);
            String token = validateUser(loginBody.getAccount(),loginBody.getPassword());
            //设置推送别名
            JpushClientUtil.bindAlias(loginBody.getRegisterId(),loginBody.getAccount());
            ajax.put(Constants.TOKEN, token);
            return ajax;
        }

        //已注册 直接登录
        String token = validateUser(loginBody.getAccount(),loginBody.getPassword());
        //设置推送别名
        JpushClientUtil.bindAlias(loginBody.getRegisterId(),loginBody.getAccount());
        if (!loginBody.getRegisterId().equals(user.getRegisterId())){
            //判断登录设备是否更换 todo
//            userService.updateUser();
        }
        //返回token
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    /**
     * 校验验证码
     *
     * @param code 验证码
     * @param uuid 唯一标识
     * @return 结果
     */
    public void validateCaptcha(String username, String code, String uuid)
    {
        String verifyKey = CacheConstants.CAPTCHA_CODE_KEY + StringUtils.nvl(uuid, "");
        String captcha = redisCache.getCacheObject(verifyKey);
        redisCache.deleteObject(verifyKey);
        if (captcha == null)
        {
            //验证码为空
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.expire")));
            throw new CaptchaExpireException();
        }
        if (!code.equalsIgnoreCase(captcha))
        {
            //验证码不存在
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.error")));
            throw new CaptchaException();
        }
    }

    //用户验证
    public String validateUser(String username,String password){
        // 用户验证
        Authentication authentication = null;
        try
        {
            UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(username, password);
            AuthenticationContextHolder.setContext(authenticationToken);
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager.authenticate(authenticationToken);
        }
        catch (Exception e)
        {
            if (e instanceof BadCredentialsException)
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
                throw new UserPasswordNotMatchException();
            }
            else
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, e.getMessage()));
                throw new ServiceException(e.getMessage());
            }
        }
        finally
        {
            AuthenticationContextHolder.clearContext();
        }
        AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_SUCCESS, MessageUtils.message("user.login.success")));
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        if (StrUtil.isEmpty(loginUser.getUser().getMerchantId())){
            throw new RuntimeException("未建商户无法登录");
        }
        recordLoginInfo(loginUser.getUserId());
        // 生成token
        return tokenService.createToken(loginUser);
    }


    /**
     * 获取用户信息
     *
     * @return 用户信息
     */
    @GetMapping("/getInfo")
    public AjaxResult getInfo()
    {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        // 角色集合
//        Set<String> roles = permissionService.getRolePermission(user);
        // 权限集合
//        Set<String> permissions = permissionService.getMenuPermission(user);
        AjaxResult ajax = AjaxResult.success();
        ajax.put("user", user);
        ajax.put("merchant",getIndex(user));
//        ajax.put("roles", roles);
//        ajax.put("permissions", permissions);
        return ajax;
    }
    /**
     * 记录登录信息
     *
     * @param userId 用户ID
     */
    public void recordLoginInfo(Long userId)
    {
        SysUser sysUser = new SysUser();
        sysUser.setUserId(userId);
        sysUser.setLoginIp(IpUtils.getIpAddr(ServletUtils.getRequest()));
        sysUser.setLoginDate(DateUtils.getNowDate());
        userService.updateUserProfile(sysUser);
    }

//    -----------------------------------------------
    @ApiOperation("获取验证码")
//    @PostMapping ("/getCode")
    public RestResponse getCode(@RequestBody Consumer consumer){
        String  captcha=(String) redisTemplate.opsForValue().get(consumer.getConsumerPhonenumber());
        if (StrUtil.isNotEmpty(captcha)){
            return new RestResponse().setSuccess(false).setMessage("请1分钟后重试");
        }
        captcha =gen()+"";

            //发送成功  手机号存到redis 对应code 1分钟有效期
            redisTemplate.opsForValue().set(consumer.getConsumerPhonenumber(),
                                                                    captcha,
                                                                    1,
                                                                    TimeUnit.MINUTES);
            return new RestResponse().setSuccess(true).setMessage("发送成功").setData(captcha);
    }

    public int gen() {
        Random r = new Random( System.currentTimeMillis() );
        return ((1 + r.nextInt(2)) * 100000 + r.nextInt(100000));
    }

    /**
     * 短信验证码登录方式
     * @param map
     * @return
     */
    public RestResponse login(@RequestBody Map<String,String> map){
        System.out.println("---------------------registerId"+map.get("registerId"));
        String  captcha=(String)redisTemplate.opsForValue().get(map.get("consumerPhonenumber"));
        if(map.get("captcha").equals(captcha)){
            //登录成功
            redisTemplate.delete(map.get("consumerPhonenumber"));
            Map<String, Object> dataMap = new HashMap<>();
            //查找数据库，判断是否为新用户:
            //老用戶
            Consumer consumer=service.getOne(new QueryWrapper<Consumer>()
                    .eq("consumer_phonenumber",map.get("consumerPhonenumber")));
            if (ObjectUtil.isNotEmpty(consumer)){
                //老客户
                if (StrUtil.isNotEmpty(map.get("registerId"))){
                    consumer.setRegisterId(map.get("registerId"));
                    service.updateById(consumer);
                    //设置推送别名
                    JpushClientUtil.bindAlias(consumer.getRegisterId(),consumer.getConsumerPhonenumber());
                }
                String token = jwtUtil1.generateToken(consumer.getConsumerId().toString(),consumer.getConsumerPhonenumber());
                dataMap.put("jwt",token);
                dataMap.put("userInfo",consumer);
                return new RestResponse().setSuccess(true).setMessage("成功").setData(dataMap);
            }
            //新用戶
            consumer=new Consumer();
            consumer.setConsumerPhonenumber(map.get("consumerPhonenumber"));
            consumer.setConsumerName("用户"+map.get("consumerPhonenumber"));
            consumer.setConsumerNickName("用户"+map.get("consumerPhonenumber"));
            consumer.setRegisterId(map.get("registerId"));
            consumer.setConsumerAvatar("/default.jpeg");
            service.save(consumer);
            //设置推送别名
            JpushClientUtil.bindAlias(consumer.getRegisterId(),consumer.getConsumerPhonenumber());
            String token = jwtUtil1.generateToken(consumer.getConsumerId().toString(),consumer.getConsumerPhonenumber());
            dataMap.put("jwt",token);
            dataMap.put("userInfo",consumer);
//            dataMap.put("index",getIndex(consumer));

            return new RestResponse().setSuccess(true).setMessage("成功").setData(dataMap);
        }

        return new RestResponse().setSuccess(false).setMessage("失败");
    }
    @Resource
    private YjMerchantServiceImpl merchantService;

    public Map<String,String> getIndex(SysUser c){
        YjMerchant merchant;
        Map m=new HashMap();
        if (StrUtil.isNotEmpty(c.getVisitStore())){
            merchant=merchantService.getById(c.getVisitStore());
        }else {
            merchant= merchantService.list().get(0);
        }
        m.put("address",merchant.getAddress());
        m.put("enterpriseName",merchant.getEnterpriseName());
        m.put("phone",merchant.getPhone());
        m.put("banner",merchant.getBanner());
        m.put("id",merchant.getId());
        return  m;
    }

}
