package com.ruoyi.web.controller.app_shop;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.ruoyi.RestResponse;
import com.ruoyi.shop.domain.ShopGoods;
import com.ruoyi.shop.domain.ShopOrder;
import com.ruoyi.shop.domain.ShopOrderItem;
import com.ruoyi.shop.service.impl.ShopCartServiceImpl;
import com.ruoyi.shop.service.impl.ShopGoodsServiceImpl;
import com.ruoyi.shop.service.impl.ShopOrderItemServiceImpl;
import com.ruoyi.shop.service.impl.ShopOrderServiceImpl;
import com.ruoyi.web.controller.app_shop.dto.ShopOrderDto;
import com.ruoyi.web.util.PayUtil;
import com.ruoyi.web.util.SnowFlake;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Api(tags = "订单信息", description = "提供订单信息相关的API")
@RestController
@RequestMapping("/api/order")
public class OrderController {

    @Resource
    private ShopGoodsServiceImpl goodsService;
    @Resource
    private ShopCartServiceImpl cartService;
    @Resource
    private ShopOrderServiceImpl orderService;
    @Resource
    private ShopOrderItemServiceImpl itemService;


    /**
     * 提交订单
     * @param orderDto
     * @param request
     * @return
     * ①按商品所属商户生成订单
     * ②生成订单详情
     * ③删除对应购物车
     * ④修改商品库存
     */
    @ApiOperation("提交订单-生成预付单")
    @PostMapping("/createOrder")
    @Transactional(rollbackFor = RuntimeException.class)
    public RestResponse createOrder(@RequestBody ShopOrderDto orderDto, HttpServletRequest request) throws Exception{
        Long userId= Long.parseLong(request.getAttribute("userId").toString());

        //商户:商品list
        Map<String,List<ShopGoods>> goodsMap=orderDto.getList().stream().collect(Collectors.groupingBy(ShopGoods::getBusinessId));
        String OrderSn=SnowFlake.getOrder()+"";
        BigDecimal total=new BigDecimal(0);//微信支付金额

        for (String key:goodsMap.keySet()){
            //生成订单
            ShopOrder order=new ShopOrder();
            order.setBusinessId(key);//商户
            order.setIdUser(userId);
            order.setConsignee(orderDto.getName());//收件人
            order.setConsigneeAddress(orderDto.getProvince()+" "+
                    orderDto.getCity()+" "+
                    orderDto.getAddress());//收件地址
            order.setMobile(orderDto.getPhone());//收件人电话
            order.setCreateTime(DateUtil.parseLocalDateTime(DateUtil.now(),"yyyy-MM-dd HH:mm:ss"));
            order.setOrderSn(OrderSn);//订单号
            order.setPayType("微信支付");
            order.setOrderType(1);//订单类型：1.商品 2.会籍 3.课程
            orderService.save(order);
            BigDecimal totalPrice=new BigDecimal(0);//总金额
            BigDecimal tradeAmount=new BigDecimal(0);//交易金额

            for (ShopGoods goods:goodsMap.get(key)){
                ShopOrderItem item=new ShopOrderItem();
                item.setGoodsName(goods.getName());//* 商品名称
                item.setIdGoods(goods.getId());//* 商品名称
                item.setThumbnail(goods.getThumbnail());//* 商品缩略图
                item.setBrief(goods.getBrief());//* 商品简介
                item.setAttribute(goods.getAttribute());//* 规格
                item.setCount(goods.getCount());//数量
                item.setPrice(goods.getPrice());//单价
                item.setIdOrder(order.getId());//订单id
                item.setType("1");//1 商品 2 会籍 3  课程
                BigDecimal to=goods.getPrice().multiply(new BigDecimal(goods.getCount()));
                item.setTotalPrice(to);//总金额
                //商品库存更改
                Boolean isEnough= goodsService.update(new UpdateWrapper<ShopGoods>()
                        .setSql("stock=stock-"+goods.getCount())
                        .eq("id",goods.getId())
                        .ge("stock",goods.getCount()));
                if(!isEnough){
                    throw new RuntimeException("商品["+goods.getName()+"]库存不足!");
//                    return new RestResponse().setSuccess(false).setMessage("商品["+goods.getName()+"]库存不足!");
                }
                itemService.save(item);
                if (StrUtil.isNotEmpty(goods.getCartId())){
                    cartService.removeById(goods.getCartId());
                }
                tradeAmount=tradeAmount.add(to);//实际需要支付的金额
                totalPrice=totalPrice.add(goods.getMarketPrice().multiply(new BigDecimal(goods.getCount())));
            }
            order.setTotalPrice(totalPrice);
            order.setTradeAmount( tradeAmount);//实际需要支付的金额
            orderService.updateById(order);
            //
            total=total.add(order.getTradeAmount());
        }

        total=total.multiply(new BigDecimal(100));
        Map map=new HashMap();
        if (total.equals(BigDecimal.ZERO)){
            //0元，直接支付成功 无需调用微信支付
            orderService.paySuccess(OrderSn,DateUtil.now(),"","0");
            map.put("timeStamp","1");
            return new   RestResponse().setSuccess(true).setMessage("成功").setData(map);
        }

        //调用微信支付 生成预付单
        map= PayUtil.creatOrder(total.setScale(1, RoundingMode.HALF_UP).intValue(),OrderSn);
        return new   RestResponse().setSuccess(true).setMessage("成功").setData(map);

    }

//    public RestResponse createOrder(@RequestBody ShopOrderDto orderDto, HttpServletRequest request){
//        Long userId= Long.parseLong(request.getAttribute("userId").toString());
//
////        List<String> goodsIds = orderDto.getList().stream().map(ShopGoods::getId).collect(Collectors.toList());
////        List<ShopGoods> goodsList = goodsMapper.selectBatchIds(goodsIds);
//        Map<String,List<ShopGoods>> goodsMap=orderDto.getList().stream().collect(Collectors.groupingBy(ShopGoods::getBusinessId));
//
//        for (String key:goodsMap.keySet()){
//            //生成订单
//            ShopOrder order=new ShopOrder();
//            order.setBusinessId(key);//商户
//            order.setIdUser(userId);
//            order.setConsignee(orderDto.getName());//收件人
//            order.setConsigneeAddress(orderDto.getProvince()+" "+
//                                        orderDto.getCity()+" "+
//                                        orderDto.getAddress());//收件地址
//            order.setMobile(orderDto.getPhone());//收件人电话
//            order.setCreateTime(DateUtil.parseLocalDateTime(DateUtil.now(),"yyyy-MM-dd HH:mm:ss"));
//            order.setOrderSn(SnowFlake.getOrder()+"");
//            orderService.save(order);
//            BigDecimal totalPrice=new BigDecimal(0);//总金额
//            BigDecimal tradeAmount=new BigDecimal(0);//交易金额
//
//           for (ShopGoods goods:goodsMap.get(key)){
//               ShopOrderItem item=new ShopOrderItem();
//               item.setGoodsName(goods.getName());//* 商品名称
//               item.setIdGoods(goods.getId());//* 商品名称
//               item.setThumbnail(goods.getThumbnail());//* 商品缩略图
//               item.setBrief(goods.getBrief());//* 商品简介
//               item.setAttribute(goods.getAttribute());//* 规格
//               item.setCount(goods.getCount());//数量
//               item.setPrice(goods.getPrice());//单价
//               item.setIdOrder(order.getId());//订单id
//               BigDecimal To=goods.getPrice().multiply(new BigDecimal(goods.getCount()));
//               item.setTotalPrice(To);//总金额
//               //商品库存更改
//                  Boolean isEnough= goodsService.update(new UpdateWrapper<ShopGoods>()
//                           .setSql("stock=stock-"+goods.getCount())
//                           .eq("id",goods.getId())
//                           .ge("stock",goods.getCount()));
//                if(!isEnough){
//                    throw new RuntimeException("商品["+goods.getName()+"]库存不足!");
////                    return new RestResponse().setSuccess(false).setMessage("商品["+goods.getName()+"]库存不足!");
//                }
//                itemService.save(item);
//                if (StrUtil.isNotEmpty(goods.getCartId())){
//                    cartService.removeById(goods.getCartId());
//                }
//               totalPrice=totalPrice.add(To);
//               tradeAmount=tradeAmount.add(goods.getMarketPrice().multiply(new BigDecimal(goods.getCount())));
//           }
//            order.setTotalPrice(tradeAmount);
//            order.setTradeAmount(totalPrice);//
//            orderService.updateById(order);
//        }
//        return new   RestResponse().setSuccess(true).setMessage("成功");
//    }

    /**
     * 取消订单（订单状态为“1已下单/待付款”）
     * ①更改订单状态
     * ②加回产品库存
     * @param order
     * @return
     */
//    @PostMapping("/cancelOrder")
    @Transactional(rollbackFor = RuntimeException.class)
    public RestResponse cancelOrder(@RequestBody ShopOrder order){
        //更改订单状态
        //查询对应订单详情
        //根据订单详情数量 更改商品库存
       Boolean isUpdate= orderService.update(new UpdateWrapper<ShopOrder>()
                .set("status",2)
                .eq("id",order.getId())
                .eq("status",1));
       if (isUpdate){
           List<ShopOrderItem> itemList=itemService.list(new QueryWrapper<ShopOrderItem>()
                   .eq("id_order",order.getId()));
           for (ShopOrderItem item:itemList){
               goodsService.update(new UpdateWrapper<ShopGoods>()
                       .setSql("stock=stock+"+item.getCount())
                       .eq("id",item.getIdGoods()));
           }
           return new RestResponse().setSuccess(true).setMessage("订单取消成功");
       }

        return new RestResponse().setSuccess(false).setMessage("该订单无法取消");
    }

    /**
     * 付款
     * @param order
     * @return
     */
//    @PostMapping("/pay")
    public RestResponse pay(@RequestBody ShopOrder order){
        if(orderService.getById(order.getId()).getPayStatus()==2){
            return new RestResponse().setSuccess(true).setMessage("该订单已支付成功，请勿重复支付！");
        }
        ShopOrder o=orderService.getById(order);
        order.setStatus(3);
        order.setPayStatus(2);
        order.setPayTime(DateUtil.now());
        order.setRealPrice(o.getTradeAmount());
        orderService.updateById(order);
        return new RestResponse().setSuccess(true).setMessage("订单支付成功");
    }

    /**
     * 签收
     * @param order
     * @return
     */
    @ApiOperation("确认收货")
    @PostMapping("/received")
    public RestResponse received(@RequestBody ShopOrder order){
        order.setStatus(6);
        orderService.updateById(order);
        return new RestResponse().setSuccess(true).setMessage("成功");
    }

    /**
     * 订单列表
      * @param request
     * @return
     */
    @ApiOperation(value = "获取订单列表", notes = "我的订单")
    @PostMapping("/orderList")
    public RestResponse orderList(HttpServletRequest request){
        Long userId= Long.parseLong(request.getAttribute("userId").toString());
        List<ShopOrder> list=orderService.list(new QueryWrapper<ShopOrder>()
                .eq("id_user",userId));
        List<ShopOrder> list1=new ArrayList<>();
        List<ShopOrder> list2=new ArrayList<>();
        List<ShopOrder> list3=new ArrayList<>();
        List<ShopOrder> list4=new ArrayList<>();
        List<ShopOrder> list6=new ArrayList<>();
        for (ShopOrder order:list){
            order.setItemList(getItemList(order.getId()));
            if (order.getStatus()==1){
                list1.add(order);
            }
            if (order.getStatus()==2){
                list2.add(order);
            }
            if (order.getStatus()==3){
                list3.add(order);
            }
            if (order.getStatus()==4){
                list4.add(order);
            }
            if (order.getStatus()==6){
                list6.add(order);
            }
        }
        Map<String,List<ShopOrder>> map=new HashMap<>();
        map.put("list",list);//全部订单
        map.put("list1",list1);//未付款
        map.put("list2",list2);//2已取消
        map.put("list3",list3);//3已付款/待发货
        map.put("list4",list4);//4已发货/待收货
        map.put("list6",list6);//6已签收
        return new RestResponse().setSuccess(true).setMessage("成功").setData(map);
    }

    /**
     * 订单详情
     * @param order
     * @return
     */
    @ApiOperation("订单详情")
    @PostMapping("/getOrder")
    public RestResponse getOrder(@RequestBody ShopOrder order){
        ShopOrder o=  orderService.getById(order.getId());
        o.setItemList(getItemList(order.getId()));
        return new RestResponse().setSuccess(true).setMessage("成功").setData(o);
    }

    public List<ShopOrderItem> getItemList(String orderId){
       return itemService.list(new QueryWrapper<ShopOrderItem>().eq("id_order",orderId));
    }

}
