package com.ruoyi.web.controller.app_shop;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.ruoyi.RestResponse;
import com.ruoyi.shop.domain.ShopAddress;
import com.ruoyi.shop.mapper.ShopAddressMapper;
import com.ruoyi.shop.service.ShopAddressService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 用户地址 前端控制器
 * </p>
 *
 * @author xn
 * @since 2022-09-29
 */
@Api(tags = "用户地址", description = "提供用户地址相关的API")
@RestController
@RequestMapping("/api/address")
public class ShopAddressController {

    @Resource
    private ShopAddressMapper mapper;
    @Resource
    private ShopAddressService service;

    @ApiOperation("新增收货地址")
    @PostMapping("/add")
    public RestResponse add(@RequestBody ShopAddress address, HttpServletRequest request){
        Long userId= Long.parseLong(request.getAttribute("userId").toString());
        address.setUserId(userId);
        if (address.getIsDefault()){
            changeDefault(userId);
        }
        mapper.insert(address);
        return new RestResponse().setSuccess(true).setMessage("成功");
    }

    @ApiOperation("收货地址列表")
    @PostMapping("/list")
    public RestResponse list(HttpServletRequest request){
        Long userId= Long.parseLong(request.getAttribute("userId").toString());
        List<String> columns=new ArrayList<>();
        columns.add("add_time");
        columns.add("is_default");
        List<ShopAddress> list=mapper.selectList(new QueryWrapper<ShopAddress>()
                .eq("user_id",userId)
                .orderByDesc(columns));
        return new RestResponse().setSuccess(true).setMessage("成功").setData(list);
    }

    @ApiOperation("修改收货地址")
    @PostMapping("/edit")
    public RestResponse edit(@RequestBody ShopAddress address, HttpServletRequest request){
        Long userId= Long.parseLong(request.getAttribute("userId").toString());
        if (address.getIsDefault()){
            changeDefault(userId);
        }
        mapper.updateById(address);
        return new RestResponse().setSuccess(true).setMessage("成功");
    }

    @ApiOperation("删除收货地址")
    @PostMapping("/del")
    public RestResponse del(@RequestBody ShopAddress address){
        mapper.deleteById(address.getId());
        return new RestResponse().setSuccess(true).setMessage("成功");
    }

    public void changeDefault(Long userId){
        service.update(new UpdateWrapper<ShopAddress>().set("is_default",false).eq("user_id",userId));
    }
}

