package com.ruoyi.web.util;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.json.JSONUtil;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.wechat.pay.contrib.apache.httpclient.WechatPayHttpClientBuilder;
import com.wechat.pay.contrib.apache.httpclient.auth.*;
import com.wechat.pay.contrib.apache.httpclient.cert.CertificatesManager;
import com.wechat.pay.contrib.apache.httpclient.exception.HttpCodeException;
import com.wechat.pay.contrib.apache.httpclient.exception.NotFoundException;
import com.wechat.pay.contrib.apache.httpclient.util.AesUtil;
import com.wechat.pay.contrib.apache.httpclient.util.PemUtil;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.security.*;
import java.util.Base64;
import java.util.*;

public class PayUtil {

//    String schema = "WECHATPAY2-SHA256-RSA2048";
//    HttpUrl httpurl = HttpUrl.parse(url);
    /**
     * 预支付订单
     * @param amount 金额
     * @param orderSn 订单号
     * @return
     * @throws Exception
     */
    public static Map creatOrder(int amount,String orderSn) throws Exception{

        PrivateKey merchantPrivateKey = PemUtil.loadPrivateKey(
                new ByteArrayInputStream(PayConstants.PRIVATE_KEY.getBytes(StandardCharsets.UTF_8)));

        //使用自动更新的签名验证器，不需要传入证书
        Verifier verifier = new AutoUpdateCertificatesVerifier(
                new WechatPay2Credentials(PayConstants.MCH_ID,
                        new PrivateKeySigner(PayConstants.MCH_SERIAL_NO, merchantPrivateKey)),
                PayConstants.API_V3KEY.getBytes(StandardCharsets.UTF_8));

        CloseableHttpClient httpClient = WechatPayHttpClientBuilder.create()
                .withMerchant(PayConstants.MCH_ID, PayConstants.MCH_SERIAL_NO, merchantPrivateKey)
                .withValidator(new WechatPay2Validator(verifier))
                .build();
        String url="https://api.mch.weixin.qq.com/v3/pay/transactions/app";
        HttpPost httpPost = new HttpPost(url);
        httpPost.addHeader("Accept", "application/json");
        httpPost.addHeader("Content-type","application/json; charset=utf-8");

        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ObjectMapper objectMapper = new ObjectMapper();

        ObjectNode rootNode = objectMapper.createObjectNode();
        rootNode.put("mchid", PayConstants.MCH_ID)
                .put("appid", PayConstants.APP_ID)
                .put("notify_url", PayConstants.NOTIFY_URL)
                .put("description", "三朵兰瑜伽")
                .put("out_trade_no",orderSn);//订单号 目前是时间戳System.currentTimeMillis()
        rootNode.putObject("amount")
                .put("total", amount)//订单金额 int类型单位为分
                .put("currency", "CNY");//CNY：人民币，境内商户号仅支持人民币。

        objectMapper.writeValue(bos, rootNode);

        httpPost.setEntity(new StringEntity(bos.toString("UTF-8"), "UTF-8"));
        CloseableHttpResponse response = httpClient.execute(httpPost);
//-----------------------------------------------------------------------------------------
        String bodyAsString = EntityUtils.toString(response.getEntity());
        System.out.println(bodyAsString);
        Map<String,String> json=(Map)JSONUtil.parse(bodyAsString);

        Long timestamp= System.currentTimeMillis();//时间戳
        String nonce= RandomUtil.randomString(32);//随机字符串
        StringBuilder builder=new StringBuilder();
       //应用id
        builder.append(PayConstants.APP_ID).append("\n");
        //时间戳
        builder.append(timestamp).append("\n");
        //随机字符串
        builder.append(nonce).append("\n");
        //预支付交易会话ID
        builder.append(json.get("prepay_id")).append("\n");

        //生成签名 RSA
        String ciphertext = sign(builder.toString().getBytes(StandardCharsets.UTF_8));
        System.out.println(ciphertext);
//---------------------------------------------------------------------------------------------
        //生成签名 MD5
//        SortedMap<String,String> sort=new TreeMap();
//        sort.put("appid",PayConstants.APP_ID);
//        sort.put("timestamp",timestamp.toString());
//        sort.put("noncestr",nonce);
//        sort.put("prepayid",node.get("prepay_id").toString());
//        String ciphertext = createSign(sort).toUpperCase();

//        String stringA=
//                "appid="+PayConstants.APP_ID
//                        +"&noncestr="+nonce
////                        +"&package="+PayConstants.PACKAGE
//                        +"&partnerid="+PayConstants.MCH_ID
//                        +"&prepayid="+json.get("prepay_id").toString()
//                        +"&timestamp="+timestamp;
//
//        System.out.println(stringA);
//        String stringSignTemp = stringA+"&key="+PayConstants.API_V3KEY;
//        String ciphertext = MD5Util.MD5Encode(stringSignTemp,"UTF-8").toUpperCase();
        System.out.println(ciphertext);
        Map map=new HashMap();
        map.put("appid",PayConstants.APP_ID);
        map.put("partnerid",PayConstants.MCH_ID);
        map.put("prepayid",json.get("prepay_id").toString());//有效期为2小时，超过2小时，商户需要使用原下单参数重新请求下单接口，获取新的prepay_id
        map.put("package",PayConstants.PACKAGE);
        map.put("noncestr",nonce);
        map.put("timestamp",timestamp);
        map.put("sign",ciphertext);

        return map;
    }

    /**
     * 生成签名RSA
     * @param message
     * @return
     */
    /*message为appId、timeStamp、nonceStr、package拼接成的字符串 计算得出paySign*/
    static String sign(byte[] message) {
        try {
            PrivateKey merchantPrivateKey = PemUtil.loadPrivateKey(PayConstants.PRIVATE_KEY);
            Signature sign = Signature.getInstance("SHA256withRSA");
            sign.initSign(merchantPrivateKey);
            sign.update(message);

            return Base64.getEncoder().encodeToString(sign.sign());
        } catch (InvalidKeyException e) {
            e.printStackTrace();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (SignatureException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 生成签名MD5
     * @param parameters
     * @return
     */
    public static String createSign(SortedMap parameters){
        StringBuffer sb = new StringBuffer();
        Set es = parameters.entrySet();//全部参与传参的参数按照accsii排序(升序)
        Iterator it = es.iterator();

        while(it.hasNext()) {
            Map.Entry entry = (Map.Entry)it.next();
            String k = (String)entry.getKey();
            Object v = entry.getValue();
            if(null != v && !"".equals(v)
                    && !"sign".equals(k) && !"key".equals(k)) {
                sb.append(k + "=" + v + "&");
            }
        }
         sb.append("key=" + PayConstants.API_V3KEY);//这里是商户那里设置的key
         System.out.println("签名字符串:"+sb.toString());
        String sign =MD5Util.MD5Encode(sb.toString(),"UTF-8");
        return sign;

    }



    /**
     *
     * @param serialNumber 请求头携带的序列号
     * @param message 报文
     * @param signature 签名
     * @return
     */
    public static boolean signVerify(String serialNumber,String message,String signature){

        try {
            PrivateKey merchantPrivateKey = PemUtil.loadPrivateKey(PayConstants.PRIVATE_KEY);
            // 获取证书管理器实例
            CertificatesManager certificatesManager = CertificatesManager.getInstance();
            // 向证书管理器增加需要自动更新平台证书的商户信息
            certificatesManager.putMerchant(PayConstants.MCH_ID, new WechatPay2Credentials(PayConstants.MCH_ID,
                            new PrivateKeySigner(PayConstants.MCH_SERIAL_NO, merchantPrivateKey)),
                    PayConstants.API_V3KEY.getBytes(StandardCharsets.UTF_8));
            // 从证书管理器中获取verifier
            Verifier verifier = certificatesManager.getVerifier(PayConstants.MCH_ID);

            return verifier.verify(serialNumber, message.getBytes(StandardCharsets.UTF_8), signature);
        } catch (IOException e) {
            e.printStackTrace();
        } catch (GeneralSecurityException e) {
            e.printStackTrace();
        } catch (HttpCodeException e) {
            e.printStackTrace();
        } catch (NotFoundException e) {
            e.printStackTrace();
        }

        return false;
    }

    /**
     * 关单时使用
     * @param body
     * @return
     */
    public static String decryptOrder(String body){

        try {
            AesUtil util=new AesUtil(PayConstants.API_V3KEY.getBytes(StandardCharsets.UTF_8));
            ObjectMapper objectMapper = new ObjectMapper();
            JsonNode node=objectMapper.readTree(body);
            JsonNode resource=node.get("resource");
            String ciphertext=resource.get("ciphertext").textValue();
            String associatedData=resource.get("associated_data").textValue();
            String nonce=resource.get("nonce").textValue();

            return util.decryptToString(associatedData.getBytes(StandardCharsets.UTF_8)
                    ,nonce.getBytes(StandardCharsets.UTF_8)
                    ,ciphertext);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }

    //关单
    public static void closeOrder(String outTradeNo) throws Exception{
        PrivateKey merchantPrivateKey = PemUtil.loadPrivateKey(
                new ByteArrayInputStream(PayConstants.PRIVATE_KEY.getBytes(StandardCharsets.UTF_8)));

        //使用自动更新的签名验证器，不需要传入证书
        Verifier verifier = new AutoUpdateCertificatesVerifier(
                new WechatPay2Credentials(PayConstants.MCH_ID,
                        new PrivateKeySigner(PayConstants.MCH_SERIAL_NO, merchantPrivateKey)),
                PayConstants.API_V3KEY.getBytes(StandardCharsets.UTF_8));

        CloseableHttpClient httpClient = WechatPayHttpClientBuilder.create()
                .withMerchant(PayConstants.MCH_ID, PayConstants.MCH_SERIAL_NO, merchantPrivateKey)
                .withValidator(new WechatPay2Validator(verifier))
                .build();
//************************************************
        HttpPost httpPost = new HttpPost("https://api.mch.weixin.qq.com/v3/pay/transactions/out-trade-no/"+outTradeNo+"/close");
        httpPost.addHeader("Accept", "application/json");
        httpPost.addHeader("Content-type","application/json; charset=utf-8");

        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ObjectMapper objectMapper = new ObjectMapper();

        ObjectNode rootNode = objectMapper.createObjectNode();
        rootNode.put("mchid",PayConstants.MCH_ID);

        objectMapper.writeValue(bos, rootNode);

        httpPost.setEntity(new StringEntity(bos.toString("UTF-8"), "UTF-8"));
        CloseableHttpResponse response = httpClient.execute(httpPost);

        //204关单成功
        System.out.println(response.getStatusLine().getStatusCode()+"----204关单成功");

//        String bodyAsString = EntityUtils.toString(response.getEntity());
//        System.out.println(bodyAsString);
    }

    /**
     * 合并订单支付（无用）
     * @param amount
     * @param orderSn
     * @return
     * @throws Exception
     */
    public static Map creatMergeOrder(int amount,String orderSn) throws Exception{

        PrivateKey merchantPrivateKey = PemUtil.loadPrivateKey(
                new ByteArrayInputStream(PayConstants.PRIVATE_KEY.getBytes(StandardCharsets.UTF_8)));

        //使用自动更新的签名验证器，不需要传入证书
        Verifier verifier = new AutoUpdateCertificatesVerifier(
                new WechatPay2Credentials(PayConstants.MCH_ID,
                        new PrivateKeySigner(PayConstants.MCH_SERIAL_NO, merchantPrivateKey)),
                PayConstants.API_V3KEY.getBytes(StandardCharsets.UTF_8));

        CloseableHttpClient httpClient = WechatPayHttpClientBuilder.create()
                .withMerchant(PayConstants.MCH_ID, PayConstants.MCH_SERIAL_NO, merchantPrivateKey)
                .withValidator(new WechatPay2Validator(verifier))
                .build();

        HttpPost httpPost = new HttpPost("https://api.mch.weixin.qq.com/v3/combine-transactions/app");
        httpPost.addHeader("Accept", "application/json");
        httpPost.addHeader("Content-type","application/json; charset=utf-8");

        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ObjectMapper objectMapper = new ObjectMapper();

        ObjectNode rootNode = objectMapper.createObjectNode();
        rootNode.put("combine_mchid", PayConstants.MCH_ID)
                .put("combine_appid", PayConstants.APP_ID)
                .put("notify_url", PayConstants.NOTIFY_URL)
                .put("combine_out_trade_no",orderSn);//订单号// 目前是时间戳System.currentTimeMillis()
        rootNode.putArray("sub_orders")
                .addObject()
                .put("total", amount)//订单金额 int类型单位为分
                .put("currency", "CNY");//CNY：人民币，境内商户号仅支持人民币。

        objectMapper.writeValue(bos, rootNode);

        httpPost.setEntity(new StringEntity(bos.toString("UTF-8"), "UTF-8"));
        CloseableHttpResponse response = httpClient.execute(httpPost);

        String bodyAsString = EntityUtils.toString(response.getEntity());
        System.out.println(bodyAsString);

        String timestamp=System.currentTimeMillis()+"";//时间戳
        String nonce= RandomUtil.randomString(32);//随机字符串
        StringBuilder builder=new StringBuilder();
        //应用id
        builder.append(PayConstants.APP_ID).append("\n");
        //时间戳
        builder.append(timestamp).append("\n");
        //随机字符串
        builder.append(nonce).append("\n");

        JsonNode node=objectMapper.readTree(bodyAsString);
        //预支付交易会话ID
        builder.append(node.get("prepay_id")).append("\n");

//        String cirphertext= RsaCryptoUtil.encryptOAEP(builder.toString(),verifier.getValidCertificate());
        String ciphertext = sign(builder.toString().getBytes(StandardCharsets.UTF_8));
        System.out.println(ciphertext);

        Map map=new HashMap();
        map.put("noncestr",nonce);
        map.put("package",PayConstants.PACKAGE);
        map.put("timeStamp",timestamp);
        map.put("signType","RSA");
        map.put("paySign",ciphertext);

        return map;
    }





}
