package com.ruoyi.shop.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 订单
 * </p>
 *
 * @author xn
 * @since 2022-09-23
 */
@ApiModel(description= "返回订单信息")
@Data
@EqualsAndHashCode(callSuper = false)
public class ShopOrder implements Serializable {

    private static final long serialVersionUID=1L;


    @TableId(value = "id", type = IdType.ASSIGN_UUID)
    private String id;

    /**
     * 创建时间/注册时间
     */
    @ApiModelProperty("创建时间")
     @JsonFormat(shape = JsonFormat.Shape.STRING, pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private LocalDateTime createTime;

    /**
     * 最后更新时间
     */
     @JsonFormat(shape = JsonFormat.Shape.STRING, pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private LocalDateTime modifyTime;

    /**
     * 管理员备注
     */
    private String adminMessage;

    /**
     * 收件人
     */
    @ApiModelProperty("收件人")
    private String consignee;

    /**
     * 收件地址
     */
    @ApiModelProperty("收件地址")
    private String consigneeAddress;

    /**
     * 收件人电话
     */
    @ApiModelProperty("收件人电话")
    private String mobile;

    /**
     * 优惠券抵扣金额
     */
    @ApiModelProperty("优惠券抵扣金额")
    private String couponPrice;

    /**
     * 快递公司
     */
    @ApiModelProperty("快递公司")
    private String express;

    /**
     * 消费者id
     */
    private Long idUser;

    /**
     * 订单备注
     */
    @ApiModelProperty("订单备注")
    private String message;

    /**
     * 订单号
     */
    @ApiModelProperty("订单号")
    private String orderSn;

    /**
     * 支付流水号
     */
    @ApiModelProperty("支付流水号")
    private String payId;

    /**
     * 支付状态1:支付中;2:已付款,3:支付失败
     */
    @ApiModelProperty("支付状态1:支付中;2:已付款,3:支付失败")
    private Integer payStatus;

    /**
     * 支付成功时间
     */
    @ApiModelProperty("支付成功时间")
    private String payTime;

    /**
     * 实付类型:alipay,wechat
     */
    @ApiModelProperty("实付类型")
    private String payType;

    /**
     * 实付金额
     */
    @ApiModelProperty("实付金额")
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal realPrice;


    /**
     * 快递单号
     */
    @ApiModelProperty("快递单号")
    private String shippingSn;

    /**
     * 出库时间
     */
    @ApiModelProperty("出库时间")
     @JsonFormat(shape = JsonFormat.Shape.STRING, pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private LocalDateTime shippingTime;

    /**
     * 状态
     * 1已下单/待付款
     * 2取消订单
     * 3已付款/待发货
     * 4已发货/待收货
     * 5拒收
     * 6已签收
     */
    @ApiModelProperty(" * 状态\n" +
            "     * 1已下单/待付款\n" +
            "     * 2取消订单\n" +
            "     * 3已付款/待发货\n" +
            "     * 4已发货/待收货\n" +
            "     * 5拒收\n" +
            "     * 6已签收")
    private Integer status;

    /**
     * 总金额
     */
    @ApiModelProperty("总金额")
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal totalPrice;

    /**
     * 交易金额
     */
    @ApiModelProperty("交易金额")
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal tradeAmount;

    /**
     * 所属商户id
     */
    @ApiModelProperty("所属商户id")
    private String businessId;

    /**
     * 订单类型：1.商品 2.会籍 3.课程
     */
    @ApiModelProperty("订单类型")
    private Integer orderType;

    @ApiModelProperty("订单明细")
    @TableField(exist = false)
    private List<ShopOrderItem> itemList;

}
