package com.ruoyi.shop.service.impl;


import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.JpushClientUtil;
import com.ruoyi.course.domain.ConsumerCourse;
import com.ruoyi.course.domain.HistoryMessage;
import com.ruoyi.course.service.ConsumerCourseServiceImpl;
import com.ruoyi.course.service.HistoryMessageServiceImpl;
import com.ruoyi.search.domain.MemberUser;
import com.ruoyi.search.service.MemberUserServicelmpl;
import com.ruoyi.shop.domain.ShopGoods;
import com.ruoyi.shop.domain.ShopOrder;
import com.ruoyi.shop.domain.ShopOrderItem;
import com.ruoyi.shop.mapper.ShopOrderMapper;
import com.ruoyi.shop.service.ShopOrderService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 * 订单 服务实现类
 * </p>
 *
 * @author xn
 * @since 2022-09-23
 */
@Service
public class ShopOrderServiceImpl extends ServiceImpl<ShopOrderMapper, ShopOrder> implements ShopOrderService {

    @Resource
    private ShopOrderItemServiceImpl itemService;
    @Resource
    private ShopGoodsServiceImpl goodsService;
    @Resource
    private HistoryMessageServiceImpl historyMessageService;
    @Resource
    private MemberUserServicelmpl memberUserServicelmpl;
    @Resource
    private ConsumerCourseServiceImpl consumerCourseService;

    /**
     * 取消订单
     * @param out_trade_no 订单号
     * @return
     */
    @Transactional(rollbackFor = RuntimeException.class)
    public void cancelOrder(String out_trade_no){
        List<ShopOrder> list = this.list(new QueryWrapper<ShopOrder>().eq("order_sn",out_trade_no));
        ShopOrder order=list.get(0);
        //1.商品订单
        // ①更改订单状态
        // ②查询对应订单详情
        // ③根据订单详情数量 更改商品库存
        if (order.getOrderType()==1){

            for (ShopOrder o:list){
                Boolean isUpdate= this.update(new UpdateWrapper<ShopOrder>()
                        .set("status",2)//2取消订单
                        .set("pay_status",3)//支付状态1:支付中;2:已付款,3:支付失败
                        .eq("id",o.getId())
                        .eq("status",1)
                        .eq("pay_status",1));

                if (isUpdate){
                    List<ShopOrderItem> itemList=itemService.list(new QueryWrapper<ShopOrderItem>()
                            .eq("id_order",o.getId()));
                    for (ShopOrderItem item:itemList){
                        goodsService.update(new UpdateWrapper<ShopGoods>()
                                .setSql("stock=stock+"+item.getCount())
                                .eq("id",item.getIdGoods()));
                    }
                }
            }
        }

        else {
            this.update(new UpdateWrapper<ShopOrder>()
                    .set("status",2)//2取消订单
                    .set("pay_status",3)//支付状态1:支付中;2:已付款,3:支付失败
                    .eq("id",order.getId())
                    .eq("status",1)
                    .eq("pay_status",1));
        }
        //推送 客户 try catch
        try {
            sendM(order.getMobile(),"订单: "+order.getOrderSn()+" 支付失败！");
        } catch (Exception e) {
            e.printStackTrace();
        }

        System.out.println("订单支付失败");
    }

    /**
     * 付款成功
     * @param out_trade_no 订单号
     * @param success_time 支付成功时间 yyyy-MM-dd HH:mm:ss
     * @param transaction_id 支付流水号=微信支付订单号
     * @param realPrice 实际支付金额（分）
     * @return
     */
    public void paySuccess(String out_trade_no,String success_time,String transaction_id,String realPrice){
        //根据订单号查找订单list
        List<ShopOrder> list = this.list(new QueryWrapper<ShopOrder>().eq("order_sn",out_trade_no));
        ShopOrder order=list.get(0);
        BigDecimal payPrice= new BigDecimal(realPrice).divide(new BigDecimal(100));//实际支付金额
        //根据订单类型 分别为：
        //1. 商品
        //① 更改订单状态
        //② 推送客户 try catch
        //③ 生成消息记录
        if (order.getOrderType()==1){
            for (ShopOrder o:list){
                o.setStatus(3);//已付款/待发货
                o.setPayStatus(2);//支付状态 支付成功
                o.setPayTime(success_time);//支付成功时间
                o.setRealPrice(o.getTradeAmount());//实际支付金额
                o.setPayId(transaction_id);//支付流水号=微信支付订单号
                this.updateById(o);
            }
            //推送 客户 try catch "订单: "+orderSn
            try {
                sendM(order.getMobile(),"订单: "+order.getOrderSn()+"支付成功");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        //2. 会籍
        // ①更改订单状态
        // ②办理会籍
        // ③推送客户+会籍顾问 try catch
        // ④生成消息记录
        if (order.getOrderType()==2){

            ShopOrderItem item=itemService.getOne(new QueryWrapper<ShopOrderItem>()
            .eq("id_order",order.getId()));
            MemberUser memberUser=new MemberUser();
            memberUser.setValidity(item.getMemberValidity());
            memberUser.setMemberId(item.getIdMember());
            memberUser.setMemberName(item.getMemberType());
            memberUser.setMemberContext(item.getBrief());
            memberUser.setMemberPrice(item.getTotalPrice());
            memberUser.setMemberPicture(item.getThumbnail());
            memberUser.setConsumerId(order.getIdUser());
            memberUser.setPrice(payPrice);//实付金额
            memberUser.setStroreId(order.getBusinessId());
            memberUser.setRecommend(item.getMemberRecommend());//推荐人 电话
           if (memberUserServicelmpl.handleMember(memberUser)) {
                throw new RuntimeException();
           }

            order.setPayStatus(2);//支付状态 支付成功
            order.setPayTime(success_time);//支付成功时间
            order.setRealPrice(payPrice);//实际支付金额
            order.setPayId(transaction_id);//支付流水号=微信支付订单号
            this.updateById(order);

            try {
                sendM(order.getMobile(),"订单: "+order.getOrderSn()+"支付成功");//推送客户消息
                sendM(item.getMemberRecommend(),"您推荐的订单: "+order.getOrderSn()+"支付成功");//推送推荐人消息
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        //3. 大咖/教练培训课
        // ① 更改订单状态
        // ② 给客户添加课程
        // ③ 推送客户 try catch
        if (order.getOrderType()==3){

            ShopOrderItem item=itemService.getOne(new QueryWrapper<ShopOrderItem>()
                    .eq("id_order",order.getId()));
            ConsumerCourse consumerCourse=new ConsumerCourse();
            consumerCourse.setConsumerId(order.getIdUser());
            consumerCourse.setCourseLogId(item.getIdSouseLog());
            consumerCourseService.save(consumerCourse);

            order.setPayStatus(2);//支付状态 支付成功
            order.setPayTime(success_time);//支付成功时间
            order.setRealPrice(payPrice);//实际支付金额
            order.setPayId(transaction_id);//支付流水号=微信支付订单号
            this.updateById(order);

            //推送 客户 try catch
            try {
                sendM(order.getMobile(),"订单: "+order.getOrderSn()+"支付成功");//推送客户消息
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        System.out.println("订单支付成功");
    }

    /**
     * 推送消息
     * @param
     * @param phone 手机号
     */
    public void sendM(String phone,String notification_title){

        String msg_content="点击查看！";
        JpushClientUtil.sendMessage(phone,notification_title,msg_content);

        //消息记录
        HistoryMessage message=new HistoryMessage();
        message.setPhone(phone);
        message.setNotificationTitle(notification_title);
        message.setMsgContent(msg_content);
        message.setSendTime(DateUtil.now());
        historyMessageService.save(message);
    }

}
